# -*- coding: utf-8 -*-
from operator import itemgetter, attrgetter

from accessor.exceptions import ResolverException
from accessor.utils import indexgetter


class Accessor(object):
    """
    Utility to resolve path on an object. Example:
    
    > obj = [{'a': 'abc'}, 1]
    > a = Accessor('0.a.upper')
    > a.resolve(obj)
    'ABC'
    """
    delimiter = '.'
    fail_silently = True
    call = True
    resolvers = (
        (itemgetter, (TypeError, AttributeError, KeyError)),
        (attrgetter, (TypeError, AttributeError)),
        (indexgetter, (TypeError, TypeError, ValueError, KeyError))
    )

    def __init__(self, path):
        self.path = path

    @property
    def bits(self):
        if self.path == '':
            return ()
        return self.path.split(self.delimiter)

    def resolve(self, obj):
        current = obj
        for bit in self.bits:
            try:
                current = self.__resolve_bit(current, bit)
            except ResolverException:
                if not self.fail_silently:
                    self.__fail(bit, current, obj)
                return None
        if self.check_callable(current):
            current = current()
        return current

    def __resolve_bit(self, obj, bit):
        for resolver, exceptions in self.resolvers:
            try:
                return resolver(bit)(obj)
            except exceptions:
                pass
        raise ResolverException()

    def __fail(self, bit, current, obj):
        raise ValueError(
            u'Failed lookup for bit "{bit}" in "{cur}" when resolving "{acc}" for "{obj}"'.format(
                bit=bit, cur=current, acc=self, obj=obj))

    def check_callable(self, current):
        """
        Hook to check whether it is safe and appropriate to call the current context object.
        
        :param current: Current context object 
        :return: True if it is safe and appropriate to call `current()`
        """
        return self.call and callable(current)

    def __str__(self):
        return self.path


A = Accessor


class NonCallingAccessor(Accessor):
    call = False


class FailLoudAccessor(Accessor):
    fail_silently = False


if __name__ == '__main__':
    obj = [{'a': 'abc'}, 1]
    a = Accessor('0.a.upper')
    print(a.resolve(obj))



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

from accessor import __version__

CLASSIFIERS = [
    'Development Status :: 2 - Pre-Alpha',
    'Framework :: Django',
    'Environment :: Plugins',
    'Intended Audience :: Developers',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'License :: OSI Approved :: MIT License'
]

REQUIREMENTS = []

setup(
    name='accessor',
    version=__version__,
    description='A simple python module for resolving paths on python objects',
    long_description=open('README.rst').read(),
    author='Veit Rückert',
    author_email='veit.rueckert@web.de',
    url='http://github.com/schwobaseggl/accessor',
    download_url='http://github.com/schwobaseggl/accessor/archive/{}.tar.gz'.format(__version__),
    packages=['accessor'],
    keywords="accessor attribute key index resolve",
    classifiers=CLASSIFIERS,
    install_requires=REQUIREMENTS,
)
